#define MOZ_SOURCE_STAMP 512efd480dac58acf9eebd5f25a76b32917ab56d
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-esr52
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-esr52/rev/512efd480dac58acf9eebd5f25a76b32917ab56d
